//@module

/*
  Copyright 2011-2014 Marvell Semiconductor, Inc.
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
      http://www.apache.org/licenses/LICENSE-2.0
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

exports.pins = {
    x: {type: "A2D"},
    y: {type: "A2D"},
    z: {type: "A2D"}
};

exports.configure = function( parameters ) {
    this.x.init();
    this.y.init();
    this.z.init();
    
    this.xOffset = 0.5;
    this.yOffset = 0.5;
    this.zOffset = 0.5;
    
    if ( "xOffset" in parameters ) this.xOffset = parameters.xOffset;
    if ( "yOffset" in parameters ) this.yOffset = parameters.yOffset;
    if ( "zOffset" in parameters ) this.zOffset = parameters.zOffset;
    
    this.smoothing = {};
    this.smoothing.x = .5;
    this.smoothing.y = .5;
    this.smoothing.z = .5;
    this.smoothing.factor = 15;
    this.smoothing.rate = 50;
    if ( "rate" in parameters) this.smoothing.rate = parameters.rate;
    this.smoothing.timestamp = Date.now(); 
}

exports.calibrate = function() {
	var x = this.x.read();
	var y =	this.y.read();
	var	z = this.z.read();

	this.xOffset = x;
	this.yOffset = y;
	this.zOffset = z;
	this.smoothing.x = 0;
	this.smoothing.y = 0;
	this.smoothing.z = 0;
	
	return true;
}

exports.read = function( smooth ) {
	var x = this.x.read() - this.xOffset;
	var y =	this.y.read() - this.yOffset;
	var	z = this.z.read() - this.zOffset;
	
	if ( smooth ){
		this.smoothing.x += (x - this.smoothing.x) / this.smoothing.factor;
		this.smoothing.y += (y - this.smoothing.y) / this.smoothing.factor;
		this.smoothing.z += (z - this.smoothing.z) / this.smoothing.factor;
		
		var time = Date.now();
		if (time - this.smoothing.timestamp >= this.smoothing.rate){
			this.smoothing.timestamp = time;
			return {x: this.smoothing.x, y: this.smoothing.y, z: this.smoothing.z};
		} 
	}else{
		return {x: x, y: y, z: z};
	}
}

exports.close = function() {
	this.x.close();
	this.y.close();
	this.z.close();
}