//@program

/*
  Copyright 2011-2014 Marvell Semiconductor, Inc.
  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at
      http://www.apache.org/licenses/LICENSE-2.0
  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
*/

//Implementation Constants
var MINANGLE = 650;
var MAXANGLE = 2350;
var RANGE = MAXANGLE - MINANGLE;
var HALFRANGE = Math.round( RANGE / 2 );
var MIDANGLE = HALFRANGE + MINANGLE;

var SLOWSPEED = 10;
var FASTSPEED = 7000;

var DEFAULTANGLE = 1500;
var DEFAULTSPEED = 500;

var MOTORPORT1 = 3;
var MOTORPORT2 = 7;
//End Implementation Constants

var speed = DEFAULTSPEED;
var angle1 = DEFAULTANGLE;
var angle2 = DEFAULTANGLE;

var startSendingMotor = false;

var SAMPLETIME = 10;

var whiteSkin = new Skin({ fill:"white" });

var main = new Container({
	left: 0, right: 0, top: 0, bottom: 0, skin: whiteSkin
});

var display = new Canvas({
	left: 10, right: 10, top: 10, bottom: 10, active: true,
	behavior: Object.create(Behavior.prototype, {
		draw: {value: function(canvas){
			var w = canvas.width;
        	var h = canvas.height;
        	var ctx = canvas.getContext('2d');
        	if (!ctx) return;
        	var hh = h/2;
        	var hw = w/2;
			ctx.clearRect( 0, 0, w, h );
			ctx.lineWidth = 3;					
			ctx.lineWidth = 3;
			ctx.beginPath();
			ctx.moveTo( 0, hh );
			ctx.lineTo( w, hh );
			ctx.closePath();
			ctx.stroke();
			ctx.beginPath();
			ctx.moveTo( hw, 0 );
			ctx.lineTo( hw, h );
			ctx.closePath();
			ctx.stroke();
			ctx.beginPath();
			ctx.arc( ( w * x ), h - ( h * y ), 5 + ( 10 * z ), 0, 2*Math.PI );
			ctx.closePath();
			ctx.fillStyle="red";
			ctx.fill();		
		}},
		onTouchEnded: {value: function(container) {
			application.invoke(new Message("/calibrate"));
		}}
	})
});

var doDualMotorControl = function(channel1, angle1, channel2, angle2){
	application.invoke(new MessageWithObject("pins:/ssc/sendDualCommand", {
		channel1: channel1,
		pulseWidth1: angle1,
		channel2: channel2,
		pulseWidth2: angle2,
	}));	
};

Handler.bind("/reset", {
	onInvoke: function(handler, message){
		
	}
});	

Handler.bind( "/data", {
	onInvoke: function ( handler, message ){
    	var j = message.requestObject;  
    	x = ( ( j.x ) * MAGNITUDE ) + .5;
    	y = ( ( j.y ) * MAGNITUDE ) + .5;
    	z = ( ( j.z ) * MAGNITUDE ) + .5;
    	display.delegate( "draw" );
    	if (startSendingMotor){
    		var xAngle = MIDANGLE + (j.x * MAGNITUDE * RANGE);
    		var yAngle = MIDANGLE + (j.y * -MAGNITUDE * RANGE);
    		doDualMotorControl(MOTORPORT1, xAngle, MOTORPORT2, yAngle);
    	}
    }
});

Handler.bind( "/calibrate", {
	onInvoke: function ( handler, message ){
		handler.invoke(new MessageWithObject( "pins:/accel/calibrate" ), Message.JSON);
		angle1 = DEFAULTANGLE;
		angle2 = DEFAULTANGLE;
		doDualMotorControl(MOTORPORT1, DEFAULTANGLE, MOTORPORT2, DEFAULTANGLE); 
	},
	onComplete: function (handler, message){
		startSendingMotor = true;
	}
});

x = .5;
y = .5;
z = .5;            
MAGNITUDE = 37;

main.add( display );
application.add( main ); 

var XPIN = 56;
var YPIN = 55;
var ZPIN = 54;

application.invoke(new MessageWithObject("pins:configure", {
	ssc: {
		require: "ssc32u",
		pins: {
			controller: { rx: 33, tx: 31 }
		}
	},
	accel: {
        require: "ADXL",
        pins: {
            x: {pin: XPIN},
            y: {pin: YPIN},
            z: {pin: ZPIN}
        }
    }
}));

//Set up a "repeat" call of the read method of the ADXL335 module to run every SAMPLETIME milliseconds
var message2 = new MessageWithObject( "pins:/accel/read?repeat=on&callback=/data&interval=" + SAMPLETIME, true );
application.invoke( message2 );